###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be
#     kept as in original distribution without any changes in all software
#     screens, especially in start-up page and the software header, even if
#     the application source code has been changed or updated or code has been
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
#
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import time
import types
import base64
import xml.dom.minidom

import csv
import StringIO

from openerp.tools import expose
from openerp.tools import redirect

import cherrypy

from openerp import rpc
from openerp import tools
from openerp import common

from openerp.controllers.base import SecuredController
from openerp.utils import TinyDict

import openerp.widgets as tw

def datas_read(ids, model, fields):
    return rpc.RPCProxy(model).export_data(ids, fields)

def export_csv(fields, result, write_title=False):
    try:
        fp = StringIO.StringIO()
        writer = csv.writer(fp)
        if write_title:
            writer.writerow(fields)
        for data in result:
            row = []
            for d in data:
                if isinstance(d, basestring):
                    d = d.replace('\n',' ').replace('\t',' ')
                    try:
                        d = d.encode('utf-8')
                    except:
                        pass
                row.append(d)

            writer.writerow(row)

        fp.seek(0)
        data = fp.read()
        fp.close()

        return data
    except IOError, (errno, strerror):
        raise common.message(_("Operation failed!\nI/O error")+"(%s)" % (errno,))

def _fields_get_all(model, views):

    def parse(root, fields):

        for node in root.childNodes:

            if node.nodeName in ('form', 'notebook', 'page', 'group', 'tree', 'hpaned', 'vpaned'):
                parse(node, fields)

            elif node.nodeName=='field':
                attrs = tools.node_attributes(node)
                name = attrs['name']

                fields[name].update(attrs)

        return fields

    proxy = rpc.RPCProxy(model)

    v1 = proxy.fields_view_get(views.get('tree', False), 'tree', rpc.session.context)
    v2 = proxy.fields_view_get(views.get('form', False), 'form', rpc.session.context)

    dom = xml.dom.minidom.parseString(v1['arch'].encode('utf-8'))
    root = dom.childNodes[0]

    f1 = parse(root, v1['fields'])

    dom = xml.dom.minidom.parseString(v2['arch'].encode('utf-8'))
    root = dom.childNodes[0]

    f2 = parse(root, v2['fields'])

    fields = {}
    fields.update(f1)
    fields.update(f2)

    return fields

class ImpEx(SecuredController):

    @expose(template="templates/exp.mako")
    def exp(self, **kw):
        params, data = TinyDict.split(kw)

        views = {}
        if params.view_mode and params.view_ids:
            for i, view in enumerate(params.view_mode):
                views[view] = params.view_ids[i]


        proxy = rpc.RPCProxy('ir.exports')
        new_list = []

        headers = [{'string' : 'Name', 'name' : 'name', 'type' : 'char'}]
        tree = tw.treegrid.TreeGrid('export_fields',
                                    model=params.model,
                                    headers=headers,
                                    url='/impex/get_fields',
                                    field_parent='relation',
                                    views=views)

        tree.show_headers = False

        view = proxy.fields_view_get(False, 'tree', rpc.session.context)
        new_list = tw.listgrid.List(name='_terp_list', model='ir.exports', view=view, ids=None,
                                       domain=[('resource', '=', params.model)],
                                       context=rpc.session.context, selectable=1, editable=False, pageable=False)


        return dict(new_list=new_list, model=params.model, ids=params.ids,
                    search_domain=params.search_domain, source=params.source,
                    tree=tree)

    @expose()
    def save_exp(self, **kw):
        params, data = TinyDict.split(kw)

        selected_list = data.get('fields')
        name = data.get('savelist_name')

        proxy = rpc.RPCProxy('ir.exports')

        if selected_list and name:
            if isinstance(selected_list, basestring):
                selected_list = [selected_list]
            proxy.create({'name' : name, 'resource' : params.model, 'export_fields' : [(0, 0, {'name' : f}) for f in selected_list]})

        raise redirect('/impex/exp', **kw)

    @expose()
    def delete_listname(self, **kw):

        params, data = TinyDict.split(kw)
        proxy = rpc.RPCProxy('ir.exports')

        proxy.unlink(params.id)

        raise redirect('/impex/exp', **kw)

    @expose('json')
    def get_fields(self, model, prefix='', name='', field_parent=None, **kw):

        is_importing = kw.get('is_importing', False)

        ids = kw.get('ids', '').split(',')
        ids = [i for i in ids if i]

        views = {}
        try:
            views = eval(kw['views'])
        except:
            pass

        fields = _fields_get_all(model, views)
        fields.update({'id': {'string': 'ID'}, 'db_id': {'string': 'Database ID'}})

        fields_order = fields.keys()
        fields_order.sort(lambda x,y: -cmp(fields[x].get('string', ''), fields[y].get('string', '')))

        records = []

        for i, field in enumerate(fields_order):

            value = fields[field]
            record = {}

            id = prefix + (prefix and '/' or '') + field
            nm = name + (name and '/' or '') + value['string']

            if is_importing and (value.get('type') not in ('reference',)) and (not value.get('readonly', False) \
                        or not dict(value.get('states', {}).get('draft', [('readonly', True)])).get('readonly', True)):

                record['id'] = id
                record['items'] = {'name' : nm}
                record['action'] = 'javascript: void(0)'
                record['target'] = None
                record['icon'] = None
                record['children'] = []
                record['required'] = value.get('required', False)

                records += [record]

            elif not is_importing:
#                if ids:
#                    record['id'] = ids[i]
#                else:
                record['id'] = id

                record['items'] = {'name' : nm}
                record['action'] = 'javascript: void(0)'
                record['target'] = None
                record['icon'] = None
                record['children'] = []

                records += [record]

            if len(nm.split('/')) < 3 and value.get('relation', False):

                if is_importing and not ((value['type'] not in ('reference',)) and (not value.get('readonly', False)) and value['type']=='one2many'):
                    continue

                ref = value.pop('relation')

                proxy = rpc.RPCProxy(ref)
                cfields = proxy.fields_get(False, rpc.session.context)
                cfields_order = cfields.keys()
                cfields_order.sort(lambda x,y: -cmp(cfields[x].get('string', ''), cfields[y].get('string', '')))

                children = []
                for j, fld in enumerate(cfields_order):
                    cid = id + '/' + fld
                    cid = cid.replace(' ', '_')

                    children += [cid]

                record['children'] = children or None
                record['params'] = {'model': ref, 'prefix': id, 'name': nm}

        records.reverse()
        return dict(records=records)


    @expose('json')
    def get_namelist(self, **kw):

        params, data = TinyDict.split(kw)

        res = []
        ids = []
        id = params.id

        res = self.get_data(params.model)

        ir_export = rpc.RPCProxy('ir.exports')
        ir_export_line = rpc.RPCProxy('ir.exports.line')

        field = ir_export.read(id)
        fields = ir_export_line.read(field['export_fields'])

        name_list = []
        ids = [f['name'] for f in fields]

        for name in ids:
            name_list += [(name, res.get(name))]

        return dict(name_list=name_list)

    def get_data(self, model):

        name = ''
        prefix = ''
        ids = []

        fields_data = {}
        proxy = rpc.RPCProxy(model)
        fields = proxy.fields_get(False, rpc.session.context)

        # XXX: in GTK client, top fields comes from Screen
        if not ids:
            f1 = proxy.fields_view_get(False, 'tree', rpc.session.context)['fields']
            f2 = proxy.fields_view_get(False, 'form', rpc.session.context)['fields']

            fields = {}
            fields.update(f1)
            fields.update(f2)

        def rec(fields):

            _fields = {}

            def model_populate(fields, prefix_node='', prefix=None, prefix_value='', level=2):
                fields_order = fields.keys()
                fields_order.sort(lambda x,y: -cmp(fields[x].get('string', ''), fields[y].get('string', '')))

                for field in fields_order:
                    fields_data[prefix_node+field] = fields[field]
                    if prefix_node:
                        fields_data[prefix_node + field]['string'] = '%s%s' % (prefix_value, fields_data[prefix_node + field]['string'])
                    st_name = fields[field]['string'] or field
                    _fields[prefix_node+field] = st_name
                    if fields[field].get('relation', False) and level>0:
                        fields2 = rpc.session.execute('object', 'execute', fields[field]['relation'], 'fields_get', False, rpc.session.context)
                        model_populate(fields2, prefix_node+field+'/', None, st_name+'/', level-1)

            model_populate(fields)

            return _fields

        return rec(fields)

    @expose(content_type="application/octet-stream")
    def export_data(self, fname, fields, export_as="csv", add_names=False, import_compat=False, **kw):

        params, data = TinyDict.split(kw)
        proxy = rpc.RPCProxy(params.model)

        if isinstance(fields, basestring):
            fields = [fields]

        ctx = {}
        ctx.update(rpc.session.context.copy())
        ctx['import_comp'] = import_compat

        domain = params.seach_domain or []

        ids = params.ids or proxy.search(domain, 0, 0, 0, ctx)
        result = datas_read(ids, params.model, fields)
        
        if result.get('warning', False):
            common.message_box(_('Export Error!'), unicode(result.get('warning', False)))
            return False
        result = result.get('datas',[])
        
        if import_compat:
            params.fields2 = fields

        if export_as == 'excel':
            #add_names = True
            pass
        return export_csv(params.fields2, result, add_names)

    @expose(template="templates/imp.mako")
    def imp(self, **kw):
        params, data = TinyDict.split(kw)

        views = {}
        if params.view_mode and params.view_ids:
            for i, view in enumerate(params.view_mode):
                views[view] = params.view_ids[i]

        headers = [{'string' : 'Name', 'name' : 'name', 'type' : 'char'}]
        tree = tw.treegrid.TreeGrid('import_fields',
                                    model=params.model,
                                    headers=headers,
                                    url='/impex/get_fields',
                                    field_parent='relation',
                                    views=views,
                                    is_importing=1)

        tree.show_headers = False

        return dict(model=params.model, source=params.source, tree=tree, fields=kw.get('fields', {}))

    @expose()
    def detect_data(self, csvfile, csvsep, csvdel, csvcode, csvskip, **kw):
        params, data = TinyDict.split(kw)

        _fields = {}
        _fields_invert = {}

        def model_populate(fields, prefix_node='', prefix=None, prefix_value='', level=2):
            def str_comp(x,y):
                if x<y: return 1
                elif x>y: return -1
                else: return 0

            fields_order = fields.keys()
            fields_order.sort(lambda x,y: str_comp(fields[x].get('string', ''), fields[y].get('string', '')))
            for field in fields_order:
                if (fields[field]['type'] not in ('reference',)) and (not fields[field].get('readonly', False)):
                    st_name = prefix_value+fields[field]['string'] or field
                    _fields[prefix_node+field] = st_name
                    _fields_invert[st_name] = prefix_node+field
                    if fields[field]['type']=='one2many' and level>0:
                        fields2 = rpc.session.execute('object', 'execute', fields[field]['relation'], 'fields_get', False, rpc.session.context)
                        model_populate(fields2, prefix_node+field+'/', None, st_name+'/', level-1)

        proxy = rpc.RPCProxy(params.model)
        fields = proxy.fields_get(False, rpc.session.context)
        model_populate(fields)

        try:
            data = csv.reader(csvfile.file, quotechar=str(csvdel), delimiter=str(csvsep))
        except:
            raise common.warning(_('Error opening .CSV file'), _('Input Error.'))

        fields = []
        word=''
        try:
            for line in data:
                for word in line:
                    word = ustr(word.decode(csvcode))
                    fields += [(_fields_invert[word], word)]
                break
        except:
            raise common.warning(_('Error processing your first line of the file.\nField %s is unknown!') % (word,), _('Import Error.'))

        kw['fields'] = fields
        return self.imp(**kw)

    @expose()
    def import_data(self, csvfile, csvsep, csvdel, csvcode, csvskip, fields=[], **kw):

        params, data = TinyDict.split(kw)

        content = csvfile.file.read()
        input=StringIO.StringIO(content)
        data = list(csv.reader(input, quotechar=str(csvdel), delimiter=str(csvsep)))[int(csvskip):]
        datas = []
        #if csv_data['combo']:

        if not isinstance(fields, list):
            fields = [fields]

        for line in data:
            datas.append(map(lambda x:x.decode(csvcode).encode('utf-8'), line))
        try:
            res = rpc.session.execute('object', 'execute', params.model, 'import_data', fields, datas)
        except Exception, e:
            raise common.warning(ustr(e), _('XML-RPC error!'))
        if res[0]>=0:
            raise common.message(_('Imported %d objects!') % (res[0],))
        else:
            d = ''
            for key,val in res[1].items():
                d+= ('\t%s: %s\n' % (ustr(key),ustr(val)))
            error = _('Unable to import this record:\n%s\nError Message:\n%s\n\n%s') % (d,res[2],res[3])
            raise common.message(unicode(error))

        return self.imp(**kw)


# vim: ts=4 sts=4 sw=4 si et




import os
import sys
from locale import getlocale

libdir = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'lib')
if os.path.exists(libdir) and libdir not in sys.path:
    sys.path.insert(0, libdir)

del os
del sys
del libdir


def ustr(value):
    """This method is similar to the builtin `str` method, except
    it will return Unicode string.

    @param value: the value to convert

    @rtype: unicode
    @return: unicode string
    """

    if isinstance(value, unicode):
        return value
    
    try: # first try without encoding
        return unicode(value)
    except:
        pass
    
    try: # then try with utf-8
        return unicode(value, 'utf-8')
    except:
        pass

    try: # then try with extened iso-8858
        return unicode(value, 'iso-8859-15')
    except:
        pass

    # else use default system locale
    return unicode(value, getlocale()[1])

__builtins__['ustr'] = ustr

import i18n
i18n.install()


# vim: ts=4 sts=4 sw=4 si et




import os
import sys
import string

from gettext import translation

#TODO: use Babel

import cherrypy

from openerp.i18n.utils import get_locale


__all__ = ['get_locale_dir', 'is_locale_supported', 'get_catalog', 'gettext', 'install']


_locale_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", "locales"))

def get_locale_dir():
    return _locale_dir

_supported = {}
def is_locale_supported(locale, domain=None):
    try:
        return _supported[str(locale)]
    except:
        pass
    domain = domain or "messages"
    localedir = get_locale_dir()
    _supported[str(locale)] = res = localedir and os.path.exists(os.path.join(
        localedir, locale, "LC_MESSAGES", "%s.mo" % domain))
    return res

_catalogs = {}

def get_catalog(locale, domain=None):
        
    domain = domain or "messages"
    
    catalog = _catalogs.setdefault(domain, {})
    messages = catalog.get(locale)
    if not messages:
        localedir = get_locale_dir()
        messages = catalog[locale] = translation(domain=domain,
            localedir=localedir, languages=[locale])
    return messages


def _gettext(key, locale=None, domain=None):
    """Get the gettext value for key.

    Added to builtins as '_'. Returns Unicode string.

    @param key: text to be translated
    @param locale: locale code to be used.
        If locale is None, gets the value provided by get_locale.

    """
    if locale is None:
        locale = get_locale()
    if not is_locale_supported(locale):
        locale = locale[:2]
    if key == '':
        return '' # special case
    try:
        return get_catalog(locale, domain).ugettext(key)
    except KeyError:
        return key
    except IOError:
        return key

class lazystring(object):
    """Has a number of lazily evaluated functions replicating a string.

    Just override the eval() method to produce the actual value.

    """

    def __init__(self, func, *args, **kw):
        self.func = func
        self.args = args
        self.kw = kw

    def eval(self):
        return self.func(*self.args, **self.kw)

    def __unicode__(self):
        return unicode(self.eval())

    def __str__(self):
        return str(self.eval())

    def __mod__(self, other):
        return self.eval() % other

    def __cmp__(self, other):
        return cmp(self.eval(), other)

    def __eq__(self, other):
        return self.eval() == other

    def __deepcopy__(self, memo):
        return self

def lazify(func):
    def newfunc(*args, **kw):
        lazystr = lazystring(func, *args, **kw)
        return lazystr
    return newfunc

_lazy_gettext = lazify(_gettext)

def gettext(key, locale=None, domain=None):
    if cherrypy.request.app:
        return _gettext(key, locale, domain)
    return _lazy_gettext(key, locale, domain)

def gettext2(key, locale=None, domain=None, **kw):
    value = gettext(key, locale, domain)
    if kw:
        try:
            return value % kw or None
        except:
            pass
    return value
    
def install():
    __builtins__['_'] = gettext2




# -*- encoding: utf-8 -*-
###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be
#     kept as in original distribution without any changes in all software
#     screens, especially in start-up page and the software header, even if
#     the application source code has been changed or updated or code has been
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
#
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import os
import time
import random
import locale
import xml.dom.minidom
import base64

import urllib
import re

import simplejson

from openerp import rpc
from openerp import tools
from openerp import common
from openerp import cache

from base import JSSource, JSLink
from interface import TinyWidget


DT_FORMAT = '%Y-%m-%d'
DHM_FORMAT = '%Y-%m-%d %H:%M:%S'
HM_FORMAT = '%H:%M:%S'

if not hasattr(locale, 'nl_langinfo'):
    locale.nl_langinfo = lambda *a: '%x'

if not hasattr(locale, 'D_FMT'):
    locale.D_FMT = None
    
    
COLOR_PALETTE = ['#f57900', '#cc0000', '#d400a8', '#75507b', '#3465a4', '#73d216', '#c17d11', '#edd400',
                 '#fcaf3e', '#ef2929', '#ff00c9', '#ad7fa8', '#729fcf', '#8ae234', '#e9b96e', '#fce94f',
                 '#ff8e00', '#ff0000', '#b0008c', '#9000ff', '#0078ff', '#00ff00', '#e6ff00', '#ffff00',
                 '#905000', '#9b0000', '#840067', '#510090', '#0000c9', '#009b00', '#9abe00', '#ffc900',]

_colorline = ['#%02x%02x%02x' % (25+((r+10)%11)*23,5+((g+1)%11)*20,25+((b+4)%11)*23) for r in range(11) for g in range(11) for b in range(11) ]
def choice_colors(n):
    if n > len(COLOR_PALETTE):
        return _colorline[0:-1:len(_colorline)/(n+1)]
    elif n:
        return COLOR_PALETTE[:n]
    return []


class Graph(TinyWidget):

    template = "templates/graph.mako"
    javascript = [JSSource("""
    var onChartClick = function(path) {
        //alert(path);
        window.location.href = path;
    }
    """),
    JSLink("openerp", "javascript/swfobject.js")]

    params = ['width', 'height', 'data']
    width = 500
    height = 350

    def __init__(self, model, view=False, view_id=False, ids=[], domain=[], context={}, width=500, height=350):

        name = 'graph_%s' % (random.randint(0,10000))
        super(Graph, self).__init__(name=name, model=model, width=width, height=height)

        ctx = rpc.session.context.copy()
        ctx.update(context or {})

        view = view or cache.fields_view_get(model, view_id, 'graph', ctx)
        
        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        root = dom.childNodes[0]
        attrs = tools.node_attributes(root)
        
        self.string = attrs.get('string')
        
        chart_type = attrs.get('type', 'pie')

        self.ids = ids
        if ids is None:
            self.ids = rpc.RPCProxy(model).search(domain, 0, 0, 0, ctx)
            
        if chart_type == "bar":
            self.data = BarChart(model, view, view_id, ids, domain, context)
        else:
            self.data = PieChart(model, view, view_id, ids, domain, context)
            
        self.data = simplejson.dumps(self.data.get_data())

class GraphData(object):

    def __init__(self, model, view=False, view_id=False, ids=[], domain=[], context={}):

        ctx = {}
        ctx = rpc.session.context.copy()
        ctx.update(context)

        view = view or cache.fields_view_get(model, view_id, 'graph', ctx)
        fields = view['fields']

        dom = xml.dom.minidom.parseString(view['arch'].encode('utf-8'))
        root = dom.childNodes[0]

        attrs = tools.node_attributes(root)

        self.model = model
        self.string = attrs.get('string', 'Unknown')
        self.kind = attrs.get('type', '')
        self.orientation = attrs.get('orientation', 'vertical')
        self.values = []

        axis, axis_data, axis_group = self.parse(root, fields)

        proxy = rpc.RPCProxy(model)

        ctx = rpc.session.context.copy()
        ctx.update(context)

        if ids is None:
            ids = proxy.search(domain, 0, 0, 0, ctx)

        rec_ids = []
        values = proxy.read(ids, fields.keys(), ctx)

        for value in values:
            res = {}
            rec_ids.append(value.get('id'))
            res['temp_id'] = value.get('id')

            for x in axis_data.keys():
                if fields[x]['type'] in ('many2one', 'char','time','text','selection'):
                    res[x] = value[x]
                    if isinstance(res[x], (list, tuple)):
                        res[x] = res[x][-1]
                    res[x] = ustr(res[x])
                elif fields[x]['type'] == 'date' and value[x]:
                    date = time.strptime(value[x], DT_FORMAT)
                    res[x] = time.strftime(locale.nl_langinfo(locale.D_FMT).replace('%y', '%Y'), date)
                elif fields[x]['type'] == 'datetime' and value[x]:
                    date = time.strptime(value[x], DHM_FORMAT)
                    if 'tz' in rpc.session.context:
                        try:
                            import pytz
                            lzone = pytz.timezone(rpc.session.context['tz'])
                            szone = pytz.timezone(rpc.session.timezone)
                            dt = DT.datetime(date[0], date[1], date[2], date[3], date[4], date[5], date[6])
                            sdt = szone.localize(dt, is_dst=True)
                            ldt = sdt.astimezone(lzone)
                            date = ldt.timetuple()
                        except:
                            pass
                    res[x] = time.strftime(locale.nl_langinfo(locale.D_FMT).replace('%y', '%Y')+' %H:%M:%S', date)
                else:
                    res[x] = float(value[x])

            if axis and isinstance(value[axis[0]], (tuple, list)):
                res['id'] = value[axis[0]][0]
            elif axis:
                res['id'] = value[axis[0]]
            else:
                res['id'] = False

            res['rec_id'] = rec_ids

            self.values.append(res)

        self.axis = axis
        self.axis_data = axis_data
        self.axis_group_field = axis_group

    def parse(self, root, fields):

        attrs = tools.node_attributes(root)

        axis = []
        axis_data = {}
        axis_group = {}

        for node in root.childNodes:
            attrs = tools.node_attributes(node)
            if node.localName == 'field':
                name = attrs['name']
                fields[name].update(attrs) # Update fields ...
                
                attrs['string'] = fields[name]['string']

                axis.append(ustr(name))
                axis_data[ustr(name)] =  attrs

        for i in axis_data:
            axis_data[i]['string'] = fields[i]['string']
            if axis_data[i].get('group', False):
                axis_group[i]=1
                axis.remove(i)

        return axis, axis_data, axis_group
    
    def get_graph_data(self):
        
        if not self.values:
            return dict(title=self.string)

        axis = self.axis
        axis_data = self.axis_data
        datas = self.values
        kind = self.kind

        operators = {
            '+': lambda x,y: x+y,
            '*': lambda x,y: x*y,
            'min': lambda x,y: min(x,y),
            'max': lambda x,y: max(x,y),
            '**': lambda x,y: x**y
        }

        keys = {}
        data_axis = {}
        label = {}

        axis_group = {}
        data_all = {}
        data_ax = []

        temp_dom = []
        label_x = []
        total_ids = []
        domain = []

        for field in axis[1:]:
            data_all = {}
            for val in datas:
                group_eval = ','.join(map(lambda x: val[x], self.axis_group_field.keys()))
                axis_group[group_eval] = 1

                key_ids = {}
                key_ids['id'] = val.get('id')
                key_ids['rec_id'] = val.get('rec_id')
                key_ids['prod_id'] = val[axis[0]]
                lbl = val[axis[0]]
                
                key_value = val[axis[0]]
                
                key = urllib.quote_plus(ustr(key_value).encode('utf-8'))
                info = data_axis.setdefault(key, {})

                data_all.setdefault(key_value, {})

                keys[key] = 1
                label[lbl] = 1

                if field in info:
                    oper = operators[axis_data[field].get('operator', '+')]
                    info[field] = oper(info[field], val[field])
                else:
                    info[field] = val[field]

                if group_eval in data_all[val[axis[0]]]:
                     oper = operators[axis_data[field].get('operator', '+')]
                     data_all[val[axis[0]]][group_eval] = oper(data_all[val[axis[0]]][group_eval], val[field])
                else:
                    data_all[val[axis[0]]][group_eval] = val[field]

                total_ids += [key_ids]
            data_ax.append(data_all)
        axis_group = axis_group.keys()
        axis_group.sort()
        keys = keys.keys()
        keys.sort()

        label = label.keys()
        label.sort()
        
        for l in label:
            x = 0
            for i in total_ids:
                if i.get('prod_id') == l and x == 0:
                    dd = i.get('id')
                    x += 1
                    temp_dom.append(dd)
            
            if not isinstance(l, basestring):
                l = ustr(l)
            
            if(len(l) > 10):
                label_x.append(l.split('/')[-1])
            else:
                label_x.append(l)
                
        for d in temp_dom:
            for val in datas:
                rec = val.get('rec_id')
            domain += [[(axis[0], '=', d), ('id', 'in', rec)]]

        values = {}
        for field in axis[1:]:
            values[field] = map(lambda x: data_axis[x][field], keys)
            
        n = len(axis)-1
        stack_list = []
        stack_id_list = []
        val = []
        grp_value = []
        
        if len(axis_group) > 1 and kind == 'bar':
            
            group_field = self.axis_group_field.keys()[0]
            new_keys = []
            for k in keys:
                k = urllib.unquote_plus(k)
                k = k.decode('utf-8')
                new_keys += [k]
                
            keys = new_keys
            for i in range(n):
                
                data = data_ax[i]
                group_data = {}
                
                for key in keys:
                    group_data[key] = {}
                    for dt in data:
                        for d in data[dt]:
                            ids = []
                            for dts in datas:
                                if dt == dts[axis[0]] and d == dts[group_field] and dt == key:
                                    ids += [dts['temp_id']]
                                    group_data[key][d] = str(data[dt][d]) + '/' + str(ids)
                
                for y in range(len(axis_group)):
                    for field in axis[1:]:
                        values[field] = [data[x].get(axis_group[y], 0.0) for x in new_keys]
                for x in new_keys:
                    for field in axis[1:]:
                        v = [data[x].get(axis_group[y], 0.0) for y in range(len(axis_group))]
                        grp_v = [group_data[x].get(axis_group[y], '0.0') for y in range(len(axis_group))]
                        val.append(v)
                        grp_value.append(grp_v)
                stack_list += val
                stack_id_list += grp_value
                
#        IF VALUES ARE ALL 0...
#        if stack_list and len(stack_list) > 0:
#            if stack_list[0] and len(stack_list[0]) > 0:
#                min_stack_val = min(stack_list[0])
#                max_stack_val = max(stack_list[0])
#            
#                if min_stack_val == max_stack_val == 0 or min_stack_val == max_stack_val == 0.0:
#                    return dict(title=self.string)

        return values, domain, self.model, label_x, axis, axis_group, stack_list, keys, axis_data, stack_id_list

class BarChart(GraphData):

    def __init__(self, model, view=False, view_id=False, ids=[], domain=[], context={}):
        super(BarChart, self).__init__(model, view, view_id, ids, domain, context)
        self.context = context

    def get_data(self):

        result = {}
        ctx =  rpc.session.context.copy()
        ctx.update(self.context)
        
        res = super(BarChart, self).get_graph_data()
        
        if len(res) > 1:
            values = res[0]
            domain = res[1]
            model = res[2]
            label_x = res[3]
            axis = res[4]
            axis_group = res[5]
            stack_list = res[6]
            stack_labels = res[7]
            axis_data = res[8]
            stack_id_list = res[9]
        else:
            return res

        def minmx_ticks(values):
            
            x_data = []
            
            for st in stack_list:
                range = 0
                for s in st:
                    range = range + s
                x_data += [range]
                
            yopts = {}
            mx = 0
            mn = 0
            tk = 2
            
            if values:
                values.sort()
                if x_data:
                    x_data.sort()
                    mx = x_data[-1]
                else:
                    mn = 0
                    mx = values[-1]

            if mx != 0:
                if mx < 0:
                    mx = mx - (10 + mx % 10)
                else:
                    mx = mx + (10 - (mx % 10))

            total = mx + mn
            tk = round(total/10)

            yopts['y_max'] = mx;
            yopts['y_min'] = mn;
            yopts['y_steps'] = tk;

            return yopts;

        temp_lbl = []
        dataset = result.setdefault('dataset', [])

        for i in label_x:
            lbl = {}
            i = re.sub(u'[êéèë]', 'e', i)
            i = re.sub(u'[ïî]', 'i', i)
            i = re.sub(u'[àâáâãä]', 'a', i)
            i = re.sub(u'[ç]', 'c', i)
            i = re.sub(u'[òóôõö]', 'o', i)
            i = re.sub(u'[ýÿ]', 'y', i)
            i = re.sub(u'[ñ]', 'n', i)
            i = re.sub(u'[ÁÂÃÄ]', 'A', i)
            i = re.sub(u'[ÈÉÊË]', 'E', i)
            i = re.sub(u'[ÌÍÎÏ]', 'I', i)
            i = re.sub(u'[ÒÓÔÕÖ]', 'O', i)
            i = re.sub(u'[ÙÚÛÜ]', 'U', i)
            i = re.sub(u'[Ý]', 'Y', i)
            i = re.sub(u'[Ñ]', 'N', i)
            
            lbl['text'] = i
            lbl['colour'] = "#432BAF"
            temp_lbl.append(lbl)

        url = []

        for x in axis[1:]:
            if len(axis_group) > 1:
                for st in stack_id_list:
                    for s in st:
                        if s.find('/') != -1:
                            ids = s.split('/')[1]
                            ids = eval(ids)
                            dom = [('id', 'in', ids)]
                            u = tools.url_plus('/form/find', _terp_view_type='tree', _terp_view_mode="['tree', 'graph']",
                               _terp_domain=ustr(dom), _terp_model=self.model, _terp_context=ustr(ctx))
        
                            url.append(u)
            
            else:
                for dom in domain:
                    u = tools.url_plus('/form/find', _terp_view_type='tree', _terp_view_mode="['tree', 'graph']",
                           _terp_domain=ustr(dom), _terp_model=self.model, _terp_context=ustr(ctx))
    
                    url.append(u)
        
        allvalues = []
        
        ChartColors = choice_colors(len(axis))
        
        for i, x in enumerate(axis[1:]):
            data = values[x]

            for j, d in enumerate(data):
                allvalues.append(d)

        yopts = minmx_ticks(allvalues)
        
        y_grid_color = True
        
        if yopts['y_steps'] == 0.0:
            yopts['y_steps'] = 1
            yopts['y_max'] = 9
            yopts['y_min'] = 0
            y_grid_color = False
            
        if y_grid_color:
            axis_y = {"steps": yopts['y_steps'], "max": yopts['y_max'], "min": yopts['y_min'],
                      "stroke": 2 }
        else:
            axis_y = {"steps": yopts['y_steps'], "max": yopts['y_max'], "min": yopts['y_min'],
                      "grid-colour": "#FFFFFF",
                      'stroke': 2 }
        
        if len(axis_group) > 1:
            ChartColors = choice_colors(len(axis_group))
            all_keys = []
            for i, x in enumerate(axis_group):
                data = {}
                data['text'] = x
                data['colour'] = ChartColors[i]
                data['font-size'] = 12
                all_keys.append(data)

            stack_val = []
            for j, stk in enumerate(stack_list):
                sval = []
                for x, s in enumerate(stk):
                    stack = {}
                    stack['val'] = s
                    if s != 0.0:
                        stack["on-click"]= "function(){onChartClick('" + url[x] + "')}"
                    stack['tip'] = s
                    sval.append(stack)
                stack_val.append(sval)
            
            result = { "elements": [{"type": "bar_stack",
                                     "colours": ChartColors,
                                     "values": [s for s in stack_val],
                                     "keys": [key for key in all_keys]}],
                        "x_axis": {"colour": "#909090",
                                   "labels": { "labels": [ lbl for lbl in stack_labels ], "rotate": "diagonal", "colour": "#ff0000"},
                                   "grid-colour" : "#FFFFFF",
                                   "3d": 3},
                        "y_axis": axis_y,
                        "bg_colour": "#FFFFFF",
                        "tooltip": {"mouse": 2 }}

        else:
            for i, x in enumerate(axis[1:]):
                datas = []
                data = values[x]
    
                for j, d in enumerate(data):
                    dt = {}
                    dt["on-click"]= "function(){onChartClick('" + url[j] + "')}"
                    dt['top'] = d
                    datas.append(dt)
                    allvalues.append(d)
    
                dataset.append({"text": axis_data[x]['string'],
                                "type": "bar_3d",
                                "colour": ChartColors[i],
                                "values": datas,
                            "font-size": 10})
            
            result = {"y_axis": axis_y,
                      "title": {"text": ""},
                      "elements": [i for i in dataset],
                      "bg_colour": "#FFFFFF",
                      "x_axis": {"colour": "#909090",
                                 "stroke": 1,
                                 "tick-height": 5,
                                 "grid-colour" : "#FFFFFF",
                                 "steps": 1, "labels": { "rotate": "diagonal", "colour": "#ff0000", "labels": [l for l in temp_lbl]},
                                 "3d": 3
                                 }
                      }

        return result


class PieChart(GraphData):

    def __init__(self, model, view=False, view_id=False, ids=[], domain=[], context={}):
        super(PieChart, self).__init__(model, view, view_id, ids, domain, context)

    def get_data(self):

        result = {}
        ctx =  rpc.session.context.copy()

        res = super(PieChart, self).get_graph_data()

        if len(res) > 1:
            values = res[0]
            domain = res[1]
            model = res[2]
            label_x = res[3]
            axis = res[4]
        else:
            return res

        dataset = result.setdefault('dataset', [])
        value = values.values()[0]
        
        url = []

        for dom in domain:
            u = tools.url_plus('/form/find', _terp_view_type='tree', _terp_view_mode="['tree', 'graph']",
                       _terp_domain=ustr(dom), _terp_model=self.model, _terp_context=ustr(ctx))

            url.append(u)

        allvalues = []
        per = []
        total_val = 0
        for i, x in enumerate(label_x):
            total_val += value[i]

        for i, x in enumerate(label_x):
            val = {}
            val['value'] = value[i]
            val['text'] = x
            val['label'] = x
            val['on-click'] = "function(){onChartClick('" + url[i] + "')}"
            
            if total_val <> 0.0:
                field_key = (100 * value[i])/total_val
                field_key = '%.2f' % (field_key)
                val["tip"] = x + ' (' + field_key + ' %)'
            else:
                val["tip"] = x

            allvalues.append(val)
            
        ChartColors = choice_colors(len(allvalues))

        dataset.append({'type': 'pie',
                        "colours": ChartColors,
                        "animate": "true",
                        "gradient-fill": 'true',
                        "no-labels": 'true',
                        "values": allvalues})

        result = {"legend": {"bg_colour": "#fefefe", 
                             "border": 'true', 
                             "position": "top", 
                             "shadow": 'true', 
                             "visible": 'true' 
                             }, 
                  "elements": [d for d in dataset],
                  "bg_colour": "#FFFFFF"}
        
        return result

# vim: ts=4 sts=4 sw=4 si et




###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be
#     kept as in original distribution without any changes in all software
#     screens, especially in start-up page and the software header, even if
#     the application source code has been changed or updated or code has been
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
#
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

import re
import random
import locale
import cherrypy
from base64 import b64encode, b64decode
from StringIO import StringIO

import wikimarkup

from openerp.widgets import JSLink, CSSLink
from openerp.widgets.form import Text

from openerp import rpc

_image = re.compile(r'img:(.*)\.(.*)', re.UNICODE)
_rss = re.compile(r'rss:(.*)\.(.*)', re.UNICODE)
_attach = re.compile(r'attach:(.*)\.(.*)', re.UNICODE)
_internalLinks = re.compile(r'\[\[.*\]\]', re.UNICODE)
_edit = re.compile(r'edit:(.*)\|(.*)', re.UNICODE)

class WikiParser(wikimarkup.Parser):

    def parse(self, text, id):
        text = text.replace('&nbsp;', 'n-b-s-p')
        text = text.replace('&amp;', 'n-a-m-p')
        text = text.replace('&','&amp;')
        text = text.replace('n-b-s-p', '&nbsp;')
        text = text.replace('n-a-m-p', '&amp;')
        text = text.replace('<code>', '<pre>')
        text = text.replace('</code>', '</pre>')

        text = wikimarkup.to_unicode(text)
        text = self.strip(text)

        text = super(WikiParser, self).parse(text)
        text = self.addImage(text, id)
        text = self.attachDoc(text, id)
        text = self.recordLink(text)
        text = self.addInternalLinks(text)
        #TODO : already implemented but we will implement it later after releasing the 5.0
        #text = self.addRss(text, id)
        return text

    def addRss(self, text, id):
        def addrss(path):
            rssurl = path.group().replace('rss:','')
            import rss.feedparser as feedparser
            data = feedparser.parse(rssurl)
            values = "<h2>%s</h2><br/>" % (data.feed.title)
            values += "%s<br/>" % (data.channel.description)
            for entry in data['entries']:
                values += "<h3><a href='%s'> %s </a></h3><br/>" % (entry.link, entry.title)
                values += "%s <br/>" % (entry.summary)

            return values

        bits = _rss.sub(addrss, text)
        return bits

    def attachDoc(self, text, id):
        def document(path):
            file = path.group().replace('attach:','')
            if file.startswith('http') or file.startswith('ftp') or file.startswith('http'):
                return "<a href='%s'>Download File</a>" % (file)
            else:
                proxy = rpc.RPCProxy('ir.attachment')
                ids = proxy.search([('datas_fname','=',file.strip()), ('res_model','=','wiki.wiki'), ('res_id','=',id)])
                if len(ids) > 0:
                    return "<a href='/wiki/getfile?file=%s&amp;id=%d'>%s</a>" % (file, id, file)
                else:
                    return """<a onclick="openWindow(getURL('/attachment', {model: 'wiki.wiki', id: %d}),
                    {name : 'Wiki Attachments'})">Attach : %s </a>""" % (id, file)
        bits = _attach.sub(document, text)
        return bits

    def addImage(self, text, id):
        def image(path):
            file = path.group().replace('img:','')
            if file.startswith('http') or file.startswith('ftp') or file.startswith('http'):
                return "<img src='%s'/>" % (file)
            else:
                proxy = rpc.RPCProxy('ir.attachment')
                ids = proxy.search([('datas_fname','=',file.strip()), ('res_model','=','wiki.wiki'), ('res_id','=',id)])
                if len(ids) > 0:
                    return "<img src='/wiki/getImage?file=%s&amp;id=%d'/>" % (file, id)
                else:
                    return """<a onclick="openWindow(getURL('/attachment', {model: 'wiki.wiki', id: %d}),
                    {name : 'Wiki Attachments'})">Attach : %s </a>""" % (id, file)
                #"[[/attachment/?model=wiki.wiki&amp;id=%d | Attach:%s]]" % (id, file)
        bits = _image.sub(image, text)
        return bits

    def recordLink(self, text):
        def record(path):
            record = path.group().replace('edit:','').split("|")
            model = record[0]
            text = record[1].replace('\r','').strip()
            label = "Edit Record"
            if len(record) > 2:
                label = record[2]
            proxy = rpc.RPCProxy(model)
            ids = proxy.name_search(text, [], '=', {})
            if len(ids):
                id = ids[0][0]
            else:
                try:
                    id = int(text)
                except:
                    id = 0
            return "[[/form/edit?model=%s&amp;id=%d | %s]]" % (model, id, label)

        bits = _edit.sub(record, text)
        return bits

    def addInternalLinks(self, text):
        from openerp import rpc
        proxy = rpc.RPCProxy('wiki.wiki')
        def link(path):
            link = path.group().replace('[','').replace('[','').replace(']','').replace(']','').split("|")

            mids = proxy.search([('name','ilike',link[0])])
            link_str = ""
            if mids:
               if len(link) == 2:
                   link_str = "<a href='/form/view?model=wiki.wiki&amp;id=%s'>%s</a>" % (mids[0], link[1])
               elif len(link) == 1:
                   link_str = "<a href='/form/view?model=wiki.wiki&amp;id=%s'>%s</a>" % (mids[0], link[0])
            else:
                if len(link) == 2:
                    link_str = "<a href='%s'>%s</a>" % (link[0], link[1])
                elif len(link) == 1:
                    link_str = "<a href='/form/edit?model=wiki.wiki&amp;id=False'>%s</a>" % (link[0])

            return link_str

        bits = _internalLinks.sub(link, text)
        return bits

def wiki2html(text, showToc, id):
    p = WikiParser(show_toc=showToc)
    return p.parse(text, id)

class WikiWidget(Text):
    template = "templates/wiki.mako"

    params = ["data"]
    css = [CSSLink('openerp', 'css/wiki.css')]
    javascript = [JSLink("openerp", "javascript/textarea.js")]

    data = None

    def set_value(self, value):
        super(WikiWidget, self).set_value(value)

        if value:
            toc = True
            id = False
            if hasattr(cherrypy.request, 'terp_record'):
                params = cherrypy.request.terp_params
                if params._terp_model == 'wiki.wiki':
                    proxy = rpc.RPCProxy('wiki.wiki')
                    toc = proxy.read([params.id], ['toc'])[0]['toc']
                    id = params.id

            text = value+'\n\n'
            html = wiki2html(text, toc, id)

            self.data = html



###############################################################################
#
# Copyright (C) 2007-TODAY Tiny ERP Pvt Ltd. All Rights Reserved.
#
# $Id$
#
# Developed by Tiny (http://openerp.com) and Axelor (http://axelor.com).
#
# The OpenERP web client is distributed under the "OpenERP Public License".
# It's based on Mozilla Public License Version (MPL) 1.1 with following
# restrictions:
#
# -   All names, links and logos of Tiny, Open ERP and Axelor must be
#     kept as in original distribution without any changes in all software
#     screens, especially in start-up page and the software header, even if
#     the application source code has been changed or updated or code has been
#     added.
#
# -   All distributions of the software must keep source code with OEPL.
#
# -   All integrations to any other software must keep source code with OEPL.
#
# If you need commercial licence to remove this kind of restriction please
# contact us.
#
# You can see the MPL licence at: http://www.mozilla.org/MPL/MPL-1.1.html
#
###############################################################################

from interface import TinyInputWidget

from form import Form
from listgrid import List

from openerp import rpc
from openerp import validators as validators

import many2one

class Reference(TinyInputWidget):

    template = "templates/reference.mako"
    params = ['options','domain','context', "text", "relation"]

    options = []

    def __init__(self, **attrs):
        super(Reference, self).__init__(**attrs)
        self.options = attrs.get('selection', [])
        
        self.domain = attrs.get('domain', [])
        self.context = attrs.get('context', {})

        self.validator = validators.Reference()
        self.onchange = None # override onchange in js code

    def set_value(self, value):
        if value:
            self.relation, self.default = value.split(",")
            self.text = many2one.get_name(self.relation, self.default)
        else:
            self.relation = ''
            self.default = ''
            self.text = ''

# vim: ts=4 sts=4 sw=4 si et

